@@ -1,3 +1,8 @@
+/* SPDX-License-Identifier: MIT
+ * Copyright (C) 2023-2025 наб (nabijaczleweli@nabijaczleweli.xyz)
+ * Copyright (C) 2023-2024 Alexandre Detiste (alexandre@detiste.be)
+ */
+
 #include "libvoreutils.hpp"
 #include "util.hpp"
 #include <md5.h>
@@ -159,6 +164,8 @@ struct Job {
 	std::size_t start_hour;
 	bool persistent;
 	bool batch;
+	std::string_view cron_batch_loadavg_below;
+	std::string_view cron_batch_throttle_group;
 	std::string jobid;
 	std::string unit_name;
 	std::string_view user;
@@ -290,6 +297,10 @@ struct Job {
 					this->log(Log::WARNING, "invalid DELAY");
 			} else if(k == "BATCH"sv)
 				this->batch = systemd_bool(v);
+			else if(k == "CRON_BATCH_LOADAVG_BELOW"sv)
+				this->cron_batch_loadavg_below = v;
+			else if(k == "CRON_BATCH_THROTTLE_GROUP"sv)
+				this->cron_batch_throttle_group = v;
 			else if(k == "CRON_MAIL_SUCCESS"sv) {
 				if(v == "never"sv || systemd_bool_false(v))
 					this->cron_mail_success = cron_mail_success_t::never;
@@ -894,9 +905,18 @@ struct Job {
 		std::fputs("SyslogFacility=cron\n", into);
 		if(USE_LOGLEVELMAX != "no"sv)
 			std::fprintf(into, "LogLevelMax=%.*s\n", FORMAT_SV(USE_LOGLEVELMAX));
+		bool have_startpre{};
 		if(!this->schedule.empty() && this->boot_delay)
 			if(!UPTIME || this->boot_delay > *UPTIME)
-				std::fprintf(into, "ExecStartPre=-%.*s %zu\n", FORMAT_SV(BOOT_DELAY), this->boot_delay);
+				std::fprintf(into, "ExecStartPre=-%s %zu\n", BOOT_DELAY, this->boot_delay), have_startpre = true;
+		if(!this->cron_batch_throttle_group.empty())
+			std::fprintf(into, "ExecStartPre=!%s %.*s%*s%.*s\n", THROTTLE_GROUP, FORMAT_SV(this->cron_batch_throttle_group), !this->cron_batch_loadavg_below.empty(),
+			             "", FORMAT_SV(this->cron_batch_loadavg_below)),
+			    have_startpre = true;
+		else if(!this->cron_batch_loadavg_below.empty())
+			std::fprintf(into, "ExecStartPre=!%s %.*s\n", LOADAVG_DAM, FORMAT_SV(this->cron_batch_loadavg_below)), have_startpre = true;
+		if(have_startpre)
+			std::fputs("TimeoutStartSec=infinity\n", into);
 		std::fprintf(into, "ExecStart=%.*s\n", FORMAT_SV(this->execstart));
 		if(onsuccess_shim) {
 			std::fputs(onsuccess_shim, into);
@@ -1094,12 +1114,8 @@ static auto parse_crontab(std::string_view filename, withuser_t withuser, std::m
 		if(!regexec(&ENVVAR_RE, line.data(), sizeof(matches) / sizeof(*matches), matches, REG_STARTEND)) {
 			auto key   = line.substr(matches[1].rm_so, matches[1].rm_eo - matches[1].rm_so);
 			auto value = line.substr(matches[2].rm_so, matches[2].rm_eo - matches[2].rm_so);
-			for(char tostrip : {'\'', '\"', ' '}) {
-				while(!value.empty() && value[0] == tostrip)
-					value.remove_prefix(1);
-				while(!value.empty() && value.back() == tostrip)
-					value.remove_suffix(1);
-			}
+			while(value.size() >= 2 && ((value[0] == '\'' && value.back() == '\'') || (value[0] == '\"' && value.back() == '\"')))
+				value.remove_prefix(1), value.remove_suffix(1);
 			if(key == "PERSISTENT"sv && value == "auto"sv)
 				environment.erase("PERSISTENT"sv);
 			else if(key == "CRON_INHERIT_VARIABLES"sv)
@@ -1304,7 +1320,7 @@ static auto is_backup(const char * path, const std::string_view & name) -> bool
 		return true;
 
 	bool backup = name[0] == '.' || name.find('~') != std::string_view::npos || name.find(".dpkg-"sv) != std::string_view::npos ||
-	              name.find(".rpm"sv) != std::string_view::npos || name == "0anacron"sv;
+	              name.find(".rpm"sv) != std::string_view::npos || name == "0anacron"sv || name == "anacron"sv;
 	if(backup)
 		log(Log::DEBUG, "ignoring %s/%.*s", path, FORMAT_SV(name));
 	return backup;
